#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Copyright (c) 2023 Blacknon. All rights reserved.
# Use of this source code is governed by an MIT license
# that can be found in the LICENSE file.
# =======================================================

"""engine_yandex
    * Yandex(yandex.com)用の検索用Classを持つモジュール.
"""

from .common import Color
from .engine_common import CommonEngine


class Yandex(CommonEngine):
    """DuckDuckGo

    DuckDuckGo用の検索エンジン用Class.
    """

    def __init__(self):
        None
